@@ -4,6 +4,7 @@
  *
  * Copyright 2001, 2002 Sun Microsystems Inc.,
  * Copyright 2001, 2002 Ximian, Inc.
+ * Copyright 2010, 2011 Novell, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -38,31 +39,6 @@ atspi_range_copy (AtspiRange *src)
 
 G_DEFINE_BOXED_TYPE (AtspiRange, atspi_range, atspi_range_copy, g_free)
 
-static AtspiRangedAttributeSet *
-atspi_ranged_attribute_set_copy (AtspiRangedAttributeSet *src)
-{
-  AtspiRangedAttributeSet *dst = g_new (AtspiRangedAttributeSet, 1);
-
-  if (dst)
-  {
-    dst->attributes = g_object_ref (src->attributes);
-    dst->start_offset = src->start_offset;
-    dst->end_offset = src->end_offset;
-  }
-  return dst;
-}
-
-static void
-atspi_ranged_attribute_set_free (AtspiRangedAttributeSet *set)
-{
-  g_object_unref (set->attributes);
-  g_free (set);
-}
-
-G_DEFINE_BOXED_TYPE (AtspiRangedAttributeSet, atspi_ranged_attribute_set,
-                    atspi_ranged_attribute_set_copy,
-                    atspi_ranged_attribute_set_free)
-
 static AtspiTextRange *
 atspi_text_range_copy (AtspiTextRange *src)
 {
@@ -70,7 +46,7 @@ atspi_text_range_copy (AtspiTextRange *src)
 
   if (dst)
   {
-    dst->text = g_strdup (src->text);
+    dst->content = g_strdup (src->content);
     dst->start_offset = src->start_offset;
     dst->end_offset = src->end_offset;
   }
@@ -80,7 +56,7 @@ atspi_text_range_copy (AtspiTextRange *src)
 static void
 atspi_text_range_free (AtspiTextRange *range)
 {
-  g_free (range->text);
+  g_free (range->content);
   g_free (range);
 }
 
@@ -127,12 +103,15 @@ atspi_text_get_text (AtspiText *obj,
                         gint end_offset,
                         GError **error)
 {
-  gchar *retval;
+  gchar *retval = NULL;
   dbus_int32_t d_start_offset = start_offset, d_end_offset = end_offset;
 
-  g_return_val_if_fail (obj != NULL, NULL);
+  g_return_val_if_fail (obj != NULL, g_strdup (""));
 
-  _atspi_dbus_call (obj, atspi_interface_text, "GetText", error, "ii=>s", start_offset, end_offset, &retval);
+  _atspi_dbus_call (obj, atspi_interface_text, "GetText", error, "ii=>s", d_start_offset, d_end_offset, &retval);
+
+  if (!retval)
+    retval = g_strdup ("");
 
   return retval;
 }
@@ -162,45 +141,48 @@ atspi_text_get_caret_offset (AtspiText *obj, GError **error)
  * @obj: a pointer to the #AtspiText object to query.
  * @offset: a long integer indicating the offset from which the attribute
  *        search is based.
+ * @start_offset: (out): a #gint indicating the start of the desired text
+ *                range.
+ * @end_offset: (out): a #gint indicating the first character past the desired
+ *              range.
  *
  * Get the attributes applied to a range of text from an #AtspiText
  *          object, and the bounds of the range.
  *          The text attributes correspond to CSS attributes where possible,
  *
- * Returns: an #AtspiRangedAttributeSet describing the attributes at the
- * given character offset
+ * Returns: (element-type gchar* gchar*) (transfer full): a #GHashTable
+ *          describing the attributes at the given character offset
  **/
-AtspiRangedAttributeSet *
+GHashTable *
 atspi_text_get_attributes (AtspiText *obj,
-			      gint offset,
-			      GError **error)
+			   gint offset,
+			   gint *start_offset,
+			   gint *end_offset,
+			   GError **error)
 {
   dbus_int32_t d_offset = offset;
   dbus_int32_t d_start_offset, d_end_offset;
-  GHashTable *hash;
   DBusMessage *reply;
   DBusMessageIter iter;
-  AtspiRangedAttributeSet *ret = g_new0 (AtspiRangedAttributeSet, 1);
-
-  if (!ret)
-    return NULL;
-  ret->start_offset = ret->end_offset = -1;
+  GHashTable *ret = NULL;
 
   if (obj == NULL)
-   return ret;
+   return NULL;
 
   reply = _atspi_dbus_call_partial (obj, atspi_interface_text, "GetAttributes", error, "i", d_offset);
-  _ATSPI_DBUS_CHECK_SIG (reply, "{ss}ii", ret)
+  _ATSPI_DBUS_CHECK_SIG (reply, "a{ss}ii", error, ret)
 
   dbus_message_iter_init (reply, &iter);
-  ret->attributes = _atspi_dbus_hash_from_iter (&iter);
+  ret = _atspi_dbus_hash_from_iter (&iter);
   dbus_message_iter_next (&iter);
 
   dbus_message_iter_get_basic (&iter, &d_start_offset);
-  ret->start_offset = d_start_offset;
+  if (start_offset)
+    *start_offset = d_start_offset;
   dbus_message_iter_next (&iter);
   dbus_message_iter_get_basic (&iter, &d_end_offset);
-  ret->end_offset = d_end_offset;
+  if (end_offset)
+    *end_offset = d_start_offset;
 
   dbus_message_unref (reply);
   return ret;
@@ -215,43 +197,47 @@ atspi_text_get_attributes (AtspiText *obj,
  *                 attributes which are explicitly set on the current attribute 
  *                 run, omitting any attributes which are inherited from the 
  *                 default values.
+ * @start_offset: (out): a #gint indicating the start of the desired text
+ *                range.
+ * @end_offset: (out): a #gint indicating the first character past the desired
+ *              range.
  *
- * Returns: the AttributeSet defined at offset, optionally including the 'default' attributes.
+ * Returns: (element-type gchar* gchar*) (transfer full): the AttributeSet
+ *          defined at offset, optionally including the 'default' attributes.
  **/
-AtspiRangedAttributeSet *
+GHashTable *
 atspi_text_get_attribute_run (AtspiText *obj,
 			      gint offset,
 			      gboolean include_defaults,
+			      gint *start_offset,
+			      gint *end_offset,
 			      GError **error)
 {
   dbus_int32_t d_offset = offset;
   dbus_int32_t d_start_offset, d_end_offset;
-  GHashTable *hash;
   DBusMessage *reply;
   DBusMessageIter iter;
-  AtspiRangedAttributeSet *ret = g_new0 (AtspiRangedAttributeSet, 1);
-
-  if (!ret)
-    return NULL;
-  ret->start_offset = ret->end_offset = -1;
+  GHashTable *ret = NULL;
 
   if (obj == NULL)
-   return ret;
+   return NULL;
 
   reply = _atspi_dbus_call_partial (obj, atspi_interface_text,
                                     "GetAttributeRun", error, "ib", d_offset,
                                     include_defaults);
-  _ATSPI_DBUS_CHECK_SIG (reply, "{ss}ii", ret)
+  _ATSPI_DBUS_CHECK_SIG (reply, "a{ss}ii", error, ret)
 
   dbus_message_iter_init (reply, &iter);
-  ret->attributes = _atspi_dbus_hash_from_iter (&iter);
+  ret = _atspi_dbus_hash_from_iter (&iter);
   dbus_message_iter_next (&iter);
 
   dbus_message_iter_get_basic (&iter, &d_start_offset);
-  ret->start_offset = d_start_offset;
+  if (start_offset)
+    *start_offset = d_start_offset;
   dbus_message_iter_next (&iter);
   dbus_message_iter_get_basic (&iter, &d_end_offset);
-  ret->end_offset = d_end_offset;
+  if (end_offset)
+    *end_offset = d_end_offset;
 
   dbus_message_unref (reply);
   return ret;
@@ -300,17 +286,14 @@ atspi_text_get_attribute_value (AtspiText *obj,
  *          (exclusive of explicitly-set attributes), encoded as UTF-8.
  **/
 GHashTable *
-atspi_text_get_default_attributes (AtspiAccessible *obj, GError **error)
+atspi_text_get_default_attributes (AtspiText *obj, GError **error)
 {
   DBusMessage *reply;
-  GHashTable *ret;
 
     g_return_val_if_fail (obj != NULL, NULL);
 
   reply = _atspi_dbus_call_partial (obj, atspi_interface_text, "GetDefaultAttributes", error, "");
-  ret = _atspi_dbus_hash_from_message (reply);
-  dbus_message_unref (reply);
-  return ret;
+  return _atspi_dbus_return_hash_from_message (reply);
 }
 
 
@@ -370,11 +353,13 @@ atspi_text_get_text_before_offset (AtspiText *obj,
     return range;
 
   _atspi_dbus_call (obj, atspi_interface_text, "GetTextBeforeOffset", error,
-                    "iu=>sii", d_offset, d_type, &range->text,
+                    "iu=>sii", d_offset, d_type, &range->content,
                     &d_start_offset, &d_end_offset);
 
   range->start_offset = d_start_offset;
   range->end_offset = d_end_offset;
+  if (!range->content)
+    range->content = g_strdup ("");
 
   return range;
 }
@@ -411,11 +396,13 @@ atspi_text_get_text_at_offset (AtspiText *obj,
     return range;
 
   _atspi_dbus_call (obj, atspi_interface_text, "GetTextAtOffset", error,
-                    "iu=>sii", d_offset, d_type, &range->text,
+                    "iu=>sii", d_offset, d_type, &range->content,
                     &d_start_offset, &d_end_offset);
 
   range->start_offset = d_start_offset;
   range->end_offset = d_end_offset;
+  if (!range->content)
+    range->content = g_strdup ("");
 
   return range;
 }
@@ -453,11 +440,13 @@ atspi_text_get_text_after_offset (AtspiText *obj,
     return range;
 
   _atspi_dbus_call (obj, atspi_interface_text, "GetTextAfterOffset", error,
-                    "iu=>sii", d_offset, d_type, &range->text,
+                    "iu=>sii", d_offset, d_type, &range->content,
                     &d_start_offset, &d_end_offset);
 
   range->start_offset = d_start_offset;
   range->end_offset = d_end_offset;
+  if (!range->content)
+    range->content = g_strdup ("");
 
   return range;
 }
@@ -550,12 +539,12 @@ atspi_text_get_offset_at_point (AtspiText *obj,
 				 GError **error)
 {
   dbus_int32_t d_x = x, d_y = y;
-  dbus_uint16_t d_type = type;
+  dbus_uint32_t d_type = type;
   dbus_int32_t retval = -1;
 
   g_return_val_if_fail (obj != NULL, -1);
 
-  _atspi_dbus_call (obj, atspi_interface_text, "GetOffsetAtPoint", error, "iin=>i", d_x, d_y, d_type, &retval);
+  _atspi_dbus_call (obj, atspi_interface_text, "GetOffsetAtPoint", error, "iiu=>i", d_x, d_y, d_type, &retval);
 
   return retval;
 }
@@ -583,7 +572,7 @@ atspi_text_get_range_extents (AtspiText *obj,
 				GError **error)
 {
   dbus_int32_t d_start_offset = start_offset, d_end_offset = end_offset;
-  dbus_int16_t d_type = type;
+  dbus_uint32_t d_type = type;
   dbus_int32_t d_x, d_y, d_width, d_height;
   AtspiRect ret;
 
@@ -592,7 +581,7 @@ atspi_text_get_range_extents (AtspiText *obj,
   if (obj == NULL)
     return atspi_rect_copy (&ret);
 
-  _atspi_dbus_call (obj, atspi_interface_text, "GetRangeExtents", error, "iin=>iiii", d_start_offset, d_end_offset, d_type, &d_x, &d_y, &d_width, &d_height);
+  _atspi_dbus_call (obj, atspi_interface_text, "GetRangeExtents", error, "iiu=>iiii", d_start_offset, d_end_offset, d_type, &d_x, &d_y, &d_width, &d_height);
 
   ret.x = d_x;
   ret.y = d_y;
@@ -639,7 +628,7 @@ atspi_text_get_bounded_ranges (AtspiText *obj,
 
   g_return_val_if_fail (obj != NULL, NULL);
 
-  _atspi_dbus_call (obj, atspi_interface_text, "GetBoundedRanges", error, "iiiinuu=>a(iisv)", d_x, d_y, d_width, d_height, d_type, d_clipTypeX, d_clipTypeY, &range_seq);
+  _atspi_dbus_call (obj, atspi_interface_text, "GetBoundedRanges", error, "iiiiuuu=>a(iisv)", d_x, d_y, d_width, d_height, d_type, d_clipTypeX, d_clipTypeY, &range_seq);
 
   return range_seq;
 }
